Package: psych
Version: 2.6.5
Date: 2026-05-11
Title: Procedures for Psychological, Psychometric, and Personality
        Research
Authors@R: person("William", "Revelle", role =c("aut","cre"), email="revelle@northwestern.edu", comment=c(ORCID  = "0000-0003-4880-9610") )
Description: A general purpose toolbox developed originally for personality, psychometric theory and experimental psychology.  Functions are primarily for multivariate analysis and scale construction using factor analysis, principal component analysis, cluster analysis and reliability analysis, although others provide basic descriptive statistics. Item Response Theory is done using  factor analysis of tetrachoric and polychoric correlations. Functions for analyzing data at multiple levels include within and between group statistics, including correlations and factor analysis.  Validation and cross validation of scales developed using basic machine learning algorithms are provided, as are functions for simulating and testing particular item and test structures. Several functions  serve as a useful front end for structural equation modeling.  Graphical displays of path diagrams, including mediation models, factor analysis and structural equation models are created using basic graphics. Some of the functions are written to support a book on psychometric theory as well as publications in personality research. For more information, see the <https://personality-project.org/r/> web page.
License: GPL (>= 2)
Imports:
        mnormt,parallel,stats,graphics,grDevices,methods,lattice,nlme,GPArotation
Suggests: psychTools, lavaan, lme4, Rcsdp, graph, knitr, Rgraphviz
LazyData: yes
ByteCompile: true
VignetteBuilder: knitr
URL: https://personality-project.org/r/psych/
        https://personality-project.org/r/psych-manual.pdf
NeedsCompilation: no
Packaged: 2026-05-15 20:20:21 UTC; WR
Author: William Revelle [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4880-9610>)
Maintainer: William Revelle <revelle@northwestern.edu>
Repository: CRAN
Date/Publication: 2026-05-16 00:10:02 UTC
Built: R 4.5.2; ; 2026-05-16 03:25:13 UTC; unix
